#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <GL/glut.h>
#include <GL/gl.h>
#include "simage.h"
#include "vector3.h"

using namespace oglclass;

float viewDistance = 8.0;
float viewRotY = 0.0;
float viewRotX = 0.0;
float viewCameraX = 0.0;
float viewCameraY = 0.0;
float prevTime = 0.0;

GLuint textureID1, textureID2;
GLUquadricObj * quadric;
float objPos[3] = {0, 0, 0};
bool objAnim = false;

GLfloat sPlaneCoefficients[4] = { 1.0, 0.0, 0.0, 0.0 };
GLfloat tPlaneCoefficients[4] = { 0.0, 1.0, 0.0, 0.0 };

void initLight(void)
{
    GLfloat white[4] = { 1, 1, 1, 1 };
    GLfloat grey20[4] = { 0.2, 0.2, 0.2, 1 };
    GLfloat lightPos[4] = {-1, 1, 2, 0};
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, white);
    glLightfv(GL_LIGHT0, GL_AMBIENT, grey20);
    glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
}

void initTexture(char *filename, GLuint& textureID)
{
	unsigned char *imageBuffer, *rescaledImage, *imagePtr;
	int imageWidth = 0, imageHeight = 0, numComponents = 0;
	imageBuffer = simage_read_image(filename, &imageWidth, &imageHeight, &numComponents);

	GLsizei xdim2,ydim2;
	GLenum type;

	xdim2 = 1;
    while (xdim2 <= imageWidth)
        xdim2 *= 2;
    xdim2 /= 2;
    ydim2 = 1;
    while (ydim2 <= imageHeight)
        ydim2 *= 2;
    ydim2 /= 2;
    if ((imageWidth != xdim2) || (imageHeight != ydim2))
        {
        rescaledImage = simage_resize(imageBuffer, imageWidth, imageHeight, numComponents,
                                           xdim2, ydim2);
        imagePtr = rescaledImage;
        }
    else
        imagePtr = imageBuffer;

	if (numComponents == 4) type = GL_RGBA;
	else if (numComponents == 3) type = GL_RGB;
	else if (numComponents == 2) type = GL_LUMINANCE_ALPHA;
	else if (numComponents == 1) type = GL_LUMINANCE;

	if (imagePtr != NULL)
	{
		glGenTextures(1, &textureID);
		glBindTexture(GL_TEXTURE_2D, textureID);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

		if (numComponents == 4)
		{
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, xdim2, ydim2,
					 0, GL_RGBA, GL_UNSIGNED_BYTE, imagePtr);
		}
		else
		{
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, xdim2, ydim2,
					 0, GL_RGB, GL_UNSIGNED_BYTE, imagePtr);
		}
		glBindTexture(GL_TEXTURE_2D, 0);
	}
}

void drawPlane(float* planeCoefficients)
{
    float A = planeCoefficients[0];
    float B = planeCoefficients[1];
    float C = planeCoefficients[2];
    float D = planeCoefficients[3];
    vector3 p0, p1, p2, p3;

    if (A != 0)
    {
        p0.set((-D+10*B+10*C)/A, -10, -10);
        p1.set((-D-10*B+10*C)/A, 10, -10);
        p2.set((-D-10*B-10*C)/A, 10, 10);
        p3.set((-D+10*B-10*C)/A, -10, 10);
    }
    else if (B != 0)
    {
        p0.set(-10, (-D+10*A+10*C)/B, -10);
        p1.set(10, (-D-10*A+10*C)/B, -10);
        p2.set(10, (-D-10*A-10*C)/B, 10);
        p3.set(-10, (-D+10*A-10*C)/B, 10);
    }
    else if (C != 0)
    {
        p0.set(-10, -10, (-D+10*A+10*B)/C);
        p1.set(10, -10, (-D-10*A+10*B)/C);
        p2.set(10, 10, (-D-10*A-10*B)/C);
        p3.set(-10, 10, (-D+10*A-10*B)/C);
    }

	vector3 u = p1 - p0;
	vector3 v = p2 - p0;
	vector3 n = vector3::crossProduct(u, v);

	glColor4f(0, 0, 1, 0.4);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glBegin(GL_QUADS);
	glNormal3fv(n.vec);
    glVertex3fv(p0.vec);
    glVertex3fv(p1.vec);
    glVertex3fv(p2.vec);
    glVertex3fv(p3.vec);
    glEnd();
	glDisable(GL_BLEND);
}

void display(void)
{
	glClearColor(0.5, 0.7, 1.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50.0, 1.0, 0.1, 100.0);
	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();

	glTranslatef(0.0, 0.0, -viewDistance);
	glRotatef(viewRotX, 1.0, 0.0, 0.0);
	glRotatef(viewRotY, 0.0, 1.0, 0.0);

	glTranslatef(objPos[0], objPos[1], objPos[2]);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textureID1);
	glEnable(GL_TEXTURE_GEN_S);
	glColor3f(1, 1, 1);
		glBegin(GL_QUADS);
		glNormal3f(0, 0, 1);
		glVertex3f(-3.5, -1, 0);
		glVertex3f(-1.5, -1, 0);
		glVertex3f(-1.5, 1, 0);
		glVertex3f(-3.5, 1, 0);
		glEnd();
	glDisable(GL_TEXTURE_GEN_S);
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textureID2);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glColor3f(1.0, 1.0, 1.0);
		glutSolidTorus(0.5, 1, 32, 24);
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textureID1);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glColor3f(1.0, 1.0, 1.0);
		glPushMatrix();
		glTranslatef(2.5, 0, 0);
		glRotatef(-90.0, 1.0, 0.0, 0.0);
		gluSphere(quadric, 1.0, 32, 24);
		glPopMatrix();
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);

	//drawPlane(sPlaneCoefficients);
	//drawPlane(tPlaneCoefficients);

	glutSwapBuffers();
}

void key(unsigned char key, int x, int y)
{
    if (key == 27)
        exit(0);
    else if (key == '+')
        viewDistance += 1;
    else if (key == '-')
        viewDistance -= 1;
	else if (key == 'a')
		objAnim = ! objAnim;
	else if (key == 'e') {
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	}
	else if (key == 'o') {
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	}
	else if (key == 's') {
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	}
    else if (key == '+') {
        sPlaneCoefficients[3] += 0.1;
        glTexGenfv(GL_S, GL_EYE_PLANE, sPlaneCoefficients);
        glTexGenfv(GL_S, GL_OBJECT_PLANE, sPlaneCoefficients);
        tPlaneCoefficients[3] += 0.1;
        glTexGenfv(GL_T, GL_EYE_PLANE, tPlaneCoefficients);
        glTexGenfv(GL_T, GL_OBJECT_PLANE, tPlaneCoefficients);
    }
    else if (key == '-') {
        sPlaneCoefficients[3] -= 0.1;
        glTexGenfv(GL_S, GL_EYE_PLANE, sPlaneCoefficients);
        glTexGenfv(GL_S, GL_OBJECT_PLANE, sPlaneCoefficients);
        tPlaneCoefficients[3] -= 0.1;
        glTexGenfv(GL_T, GL_EYE_PLANE, tPlaneCoefficients);
        glTexGenfv(GL_T, GL_OBJECT_PLANE, tPlaneCoefficients);
    }
    glutPostRedisplay();
}

void specialkey(int key, int x, int y)
{
    if (key == GLUT_KEY_LEFT)
        viewRotY += 3;
    else if (key == GLUT_KEY_RIGHT)
        viewRotY -= 3;
    else if (key == GLUT_KEY_UP)
        viewRotX += 3;
    else if (key == GLUT_KEY_DOWN)
        viewRotX -= 3;
    glutPostRedisplay();
}

void update()
{
	static int currentTime, deltaTime, prevTime = 0;
	currentTime = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = currentTime - prevTime;
	prevTime = currentTime;
	if (objAnim) {
		objPos[0] = sinf((float) currentTime * 0.001);
		objPos[1] = cosf((float) currentTime * 0.001);
	}
	glutPostRedisplay();
}

int main(int argc, char *argv[])
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowSize(512,512);
    glutCreateWindow(argv[0]);
    
    glutDisplayFunc(display);
    glutKeyboardFunc(key);
    glutSpecialFunc(specialkey);
	glutIdleFunc(update);

    initLight();
	quadric = gluNewQuadric();
    gluQuadricTexture(quadric, GL_TRUE); 
    initTexture("line.jpg", textureID1);
    initTexture("cloud.jpg", textureID2);
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
   
    glutMainLoop();
    return 0;
}

