#ifndef __VECTOR3_H__
#define __VECTOR3_H__

#include <iostream>
using namespace std;

namespace oglclass
{

class vector3
{
public:
    float vec[3];

	// constructor
	vector3(void) { vec[0] = vec[1] = vec[2] = 0.0f; }
    vector3(const vector3 &other);
	vector3(const float *other);
    vector3(float x_, float y_, float z_);
    void set(float x_, float y_, float z_);

	// length & normalize
    float length(void);
    void normalize(void);

	// static utility methods
    static float distance(const vector3 &v1, const vector3 &v2);
    static float dotProduct(const vector3 &v1,  const vector3 &v2 );
    static vector3 crossProduct(const vector3 &v1, const vector3 &v2);

    // assignment operators...
    vector3& operator = (const vector3 &other);
    vector3& operator += (const vector3 &other);
    vector3& operator -= (const vector3 &other);
    vector3& operator *= (const float scalar);
    vector3& operator /= (const float scalar);

	// equality operators
	bool operator == (const vector3 &other) const;
	bool operator != (const vector3 &other) const;

	// vector operators
    vector3 operator + (const vector3 &other);
    vector3 operator - (const vector3 &other);
    vector3 operator * (const vector3 &other);
    vector3 operator / (const vector3 &other);

    vector3 operator * (const float scalar);
    friend vector3 operator * (const float scalar, const vector3 &other);
    vector3 operator / (const float scalar);
    friend vector3 operator / (const float scalar, const vector3 &other);
    
    vector3 operator + (void) const;
    vector3 operator - (void) const;

	float& operator[] (int i);
};

inline std::ostream& operator<< (std::ostream& s, const vector3& v)
{ 
	return s << "(" << v.vec[0] << ", " << v.vec[1] << ", " << v.vec[2] << ")";
}

} // namespace

#endif
